%if 0%{?fedora}
%global with_python3 1
%endif

%global with_check 1

# SuperLUMT is on these arches only
ExcludeArch: %{power64} aarch64

%global pname assimulo
%global checkout 859
%global date     20180113

Name: python-%{pname}
Version: 2.9
Release: 23.%{date}svn%{checkout}%{?dist}
Summary: Ordinary differential and differential algebraic equations solver
License: LGPLv3+ and LGPLv2+ and BSD and GPLv3
URL: http://www.jmodelica.org/assimulo

## Source archive made by using following commands
## svn checkout https://svn.jmodelica.org/assimulo/trunk
## rm -rf trunk/.svn
## tar -cvzf assimulo-commit-%%{checkout}.tar.gz trunk
Source0: assimulo-commit-%{checkout}.tar.gz

BuildRequires: openblas-srpm-macros

%ifarch %{openblas_arches}
BuildRequires: openblas-devel, openblas-srpm-macros, openblas-static, lapack-devel, lapack-static
%else
BuildRequires: blas-devel, blas-static
BuildRequires: lapack-devel, lapack-static
%endif
BuildRequires: SuperLUMT-devel
BuildRequires: qhull-devel
BuildRequires: gcc-gfortran, gcc-c++, dos2unix
BuildRequires: sundials-devel

##This patch changes the tolerance in the mechanical systems test
##on PPC architecture
##Test failed on PPC, like this one: 
##https://trac.jmodelica.org/assimulo/ticket/239
Patch0: %{pname}-mech_system_pendulum.patch

Patch1: python-%{pname}-set_superlu_dir.patch
Patch2: python-%{pname}-set_sundials_dir.patch

# ValueError: invalid version number '1.14.0rc1'
Patch3: python-%{pname}-fix_numpy_rc_version.patch

%description
Assimulo is a Cython/Python based simulation package that allows for simulation 
of both ordinary differential equations (ODEs), f(t,y), 
and differential algebraic equations (DAEs), f(t,y,yd). 
It combines a variety of different solvers written in C, FORTRAN and 
Python via a common high-level interface.

%package -n python2-%{pname}
Summary:  Ordinary differential and differential algebraic equations solver
%{?python_provide:%python_provide python2-%{pname}}

BuildRequires: python2-devel, python2-setuptools, python2-nose
BuildRequires: python2-numpy, python2-pytest, python2-Cython, python2-matplotlib
BuildRequires: python2-numpy-f2py, python2-scipy
Requires:      python2-matplotlib, python2-numpy, python2-scipy

%description -n python2-%{pname}
Assimulo is a Cython/Python based simulation package that allows for simulation 
of both ordinary differential equations (ODEs), f(t,y), 
and differential algebraic equations (DAEs), f(t,y,yd). 
It combines a variety of different solvers written in C, FORTRAN and 
Python via a common high-level interface.

%if 0%{?with_python3}
%package -n python3-%{pname}
Summary: Ordinary differential and differential algebraic equations solver
%{?python_provide:%python_provide python3-%{pname}}

BuildRequires: python3-devel, python3-setuptools, python3-nose
BuildRequires: python3-numpy, python3-pytest, python3-Cython, python3-matplotlib
BuildRequires: python3-numpy-f2py, python3-ipython-console, python3-scipy
Requires:      python3-matplotlib, python3-numpy, python3-scipy

%description -n python3-%{pname}
%{summary}.
%endif # with_python3

%prep
%setup -qc
mv trunk python2

pushd python2
dos2unix */*
%patch0 -p1
%patch1 -p0
%patch2 -p0
%if 0%{?fedora} > 27
%patch3 -p0
%endif

sed -i 's|/usr/local/include|%{_includedir}|g' setup.py
sed -i 's|/usr/local/lib|%{_libdir}|g' setup.py

sed -i 's|@@lib@@|%{_libdir}|g' setup.py
sed -i 's|@@inc@@|%{_includedir}/SuperLUMT|g' setup.py
popd

%if 0%{?with_python3}
cp -a python2 python3
%endif # with_python3

%build
pushd python2
sed -i 's|1s|^#!python|#!%{__python2}|g' setup.py

%ifarch %{openblas_arches}
export BLASNAME=openblas
export INCBLAS=-I%{_includedir}/openblas
%else
export BLASNAME=
export INCBLAS=-I%{_includedir}
%endif

CFLAGS="%{optflags} -Wl,--as-needed" %{__python2} setup.py build --executable="%{__python2} -s" \
 --blas-home=%{_libdir} --blas-name=$BLASNAME --lapack-home=%{_libdir} \
 --sundials-home=%{_prefix} --superlu-home=%{_libdir} \
 --with_openmp=true \
 --extra-c-flags="$INCBLAS -I%{_includedir}/SuperLUMT" \
 --extra-fortran-link-flags="%{__global_ldflags}" \
 --log=DEBUG
popd

%if 0%{?with_python3}
pushd python3
sed -i 's|1s|^#!python|#!%{__python3}|g' setup.py

%ifarch %{openblas_arches}
export BLASNAME=openblas
export INCBLAS=-I%{_includedir}/openblas
%else
export BLASNAME=blas
export INCBLAS=-I%{_includedir}
%endif

CFLAGS="%{optflags} -lm -Wl,--as-needed" %{__python3} setup.py build --executable="%{__python3} -s" \
 --blas-home=%{_libdir} --blas-name=$BLASNAME --lapack-home=%{_libdir} \
 --sundials-home=%{_prefix} --superlu-home=%{_libdir} \
 --with_openmp=true \
 --extra-c-flags="$INCBLAS -I%{_includedir}/SuperLUMT" \
 --extra-fortran-link-flags="%{__global_ldflags}" \
 --log=DEBUG
popd
%endif # with_python3

%install
pushd python2
%{__python2} setup.py install --skip-build --root $RPM_BUILD_ROOT --blas-home=%{_libdir} --sundials-home=%{_prefix} --lapack-home=%{_libdir}
popd

%if 0%{?with_python3}
pushd python3
%{__python3} setup.py install --skip-build --root $RPM_BUILD_ROOT --blas-home=%{_libdir} --sundials-home=%{_prefix} --lapack-home=%{_libdir}
popd
%endif

##Fix -x permissions
##Fix wrong-script-end-of-line-encoding errors
for i in `find $RPM_BUILD_ROOT%{python2_sitearch}/%{pname} -type f \( -name "*.py" \)`; do
 chmod a+x $i
 dos2unix $i
 sed -i 's|/usr/bin/env python|%{__python2}|' $i
done

for i in `find $RPM_BUILD_ROOT%{python2_sitearch}/%{pname} -type f \( -name "*.so" \)`; do
 chmod 0755 $i
done

%if 0%{?with_python3}
for i in `find $RPM_BUILD_ROOT%{python3_sitearch}/%{pname} -type f \( -name "*.py" \)`; do
 chmod a+x $i
 dos2unix $i
 sed -i 's|/usr/bin/env python|%{__python3}|' $i
done

for i in `find $RPM_BUILD_ROOT%{python3_sitearch}/%{pname} -type f \( -name "*.so" \)`; do
 chmod 0755 $i
done
%endif

%if 0%{?with_check}
%check
# test_dasp3_basic disabled
# See http://www.jmodelica.org/27779
pushd python2/tests
%ifarch %{openblas_arches}
export BLASLIB=libopenblas.so
export LAPACKLIB=
%else
export BLASLIB=libblas.so
export LAPACKLIB=liblapack.so
%endif
rm -f __init__.py
find . -name '*.pyc' -delete
LD_PRELOAD=%{_libdir}/$BLASLIB:%{_libdir}/$LAPACKLIB
LD_PRELOAD=%{_libdir}/libsundials_idas.so
LD_PRELOAD=%{_libdir}/libsundials_cvodes.so
export PYTHONPATH=$RPM_BUILD_ROOT%{python2_sitearch}
%if 0%{?rhel}
xvfb-run -a nosetests -v --exclude=test_cvode_with_jac_sparse \
%else
nosetests -v --exclude=test_cvode_with_jac_sparse \
%endif
%ifarch %{arm}
 --exclude=test_glimda
%endif

popd

%if 0%{?with_python3}
pushd python3/tests
%ifarch %{openblas_arches}
export BLASLIB=libopenblas.so
export LAPACKLIB=
%else
export BLASLIB=libblas.so
export LAPACKLIB=liblapack.so
%endif
rm -f __init__.py
find . -name '*.pyc' -delete
LD_PRELOAD=%{_libdir}/$BLASLIB:%{_libdir}/$LAPACKLIB
LD_PRELOAD=%{_libdir}/libsundials_idas.so
LD_PRELOAD=%{_libdir}/libsundials_cvodes.so
export PYTHONPATH=$RPM_BUILD_ROOT%{python3_sitearch}
%if 0%{?rhel}
xvfb-run -a nosetests-%{python3_version} -v --exclude=test_cvode_with_jac_sparse \
%else
nosetests-%{python3_version} -v --exclude=test_cvode_with_jac_sparse \
%endif
%ifarch %{arm}
 --exclude=test_glimda
%endif

popd
%endif
%endif

%files -n python2-%{pname}
%doc python2/README python2/CHANGELOG
%license python2/thirdparty/*/LICENSE_* python2/LICENSE
%{python2_sitearch}/*.egg-info
%{python2_sitearch}/assimulo/

%if 0%{?with_python3}
%files -n python3-%{pname}
%doc python3/README python3/CHANGELOG
%license python3/thirdparty/*/LICENSE_* python3/LICENSE
%{python3_sitearch}/*.egg-info
%{python3_sitearch}/assimulo/
%endif

%changelog
* Fri Feb 09 2018 Fedora Release Engineering <releng@fedoraproject.org> - 2.9-23.20180113svn859
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Fri Feb 02 2018 Antonio Trande <sagitterATfedoraproject.org> - 2.9-22.20180113svn859
- Rebuild for GCC-8

* Sun Jan 28 2018 Antonio Trande <sagitterATfedoraproject.org> - 2.9-21.20180113svn859
- Rebuild for sundials-3.1.0

* Sun Jan 28 2018 Antonio Trande <sagitterATfedoraproject.org> - 2.9-20.20180113svn859
- Update to commit #859
- Exclude test_glimda on ARM (jmodelica.org/27939)

* Fri Dec 29 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-19.20171229svn858
- Update to commit #858 (patched for using openblas)

* Thu Dec 28 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-18.20171227svn857
- Update to commit #857 (patched for newest Sundials releases)

* Sat Dec 23 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-17.20171222svn845
- Fix Python interpreter

* Fri Dec 22 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-16.20171222svn845
- Rebuild for sundials-3*
- Update to commit #845 (patched for newest Sundials releases)
- Rebuild agaist openblas if possible

* Wed Aug 16 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-15
- Fix Summary tag

* Thu Aug 03 2017 Fedora Release Engineering <releng@fedoraproject.org> - 2.9-14
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild

* Thu Jul 27 2017 Fedora Release Engineering <releng@fedoraproject.org> - 2.9-13
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild

* Wed Mar 08 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-12
- Rebuild for sundials-2.7.0-10 again (on Fedora)

* Wed Mar 08 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-11
- Patched on Fedora > 25

* Wed Mar 08 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-10
- Rebuild for sundials-2.7.0

* Sat Feb 11 2017 Fedora Release Engineering <releng@fedoraproject.org> - 2.9-9
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_Mass_Rebuild

* Tue Jan 31 2017 Antonio Trande <sagitterATfedoraproject.org> - 2.9-8
- Rebuild for gcc-gfortran

* Mon Dec 19 2016 Miro Hrončok <mhroncok@redhat.com> - 2.9-7
- Rebuild for Python 3.6

* Thu Nov 10 2016 Antonio Trande <sagitterATfedoraproject.org> - 2.9-6
- Fix s390 builds

* Thu Oct 06 2016 Antonio Trande <sagitterATfedoraproject.org> - 2.9-5
- Rebuild for new Numpy version

* Thu Sep 29 2016 Antonio Trande <sagitterATfedoraproject.org> - 2.9-4
- Rebuild for Sundials 2.7.0
- Patched for detecting Numpy 'rc' release

* Tue Aug 16 2016 Antonio Trande <sagitterATfedoraproject.org> - 2.9-3
- Rebuild for Python 3.5.2

* Tue Jul 19 2016 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.9-2
- https://fedoraproject.org/wiki/Changes/Automatic_Provides_for_Python_RPM_Packages

* Sat Apr 30 2016 Antonio Trande <sagitterATfedoraproject.org> - 2.9-1
- Update to 2.9
- Built without SuperLU support yet

* Thu Feb 04 2016 Fedora Release Engineering <releng@fedoraproject.org> - 2.8-9
- Rebuilt for https://fedoraproject.org/wiki/Fedora_24_Mass_Rebuild

* Sun Jan 10 2016 Antonio Trande <sagitterATfedoraproject.org> - 2.8-8
- Fixed python2 package

* Thu Dec 10 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.8-7
- SPEC file adapted to recent guidelines for Python

* Tue Dec 08 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.8-6
- test_dasp3_basic skipped on > F23

* Sat Nov 14 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.8-5
- Package's name changed to python-assimulo

* Tue Nov 10 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.8-4
- Removed pname custom macro

* Tue Nov 10 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.8-3
- Fixed flags for hardened builds

* Fri Oct 02 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.8-2
- Fixed numpy's version detection
- Fixed test error on AARCH64
- Set Fortran compiler flags

* Sat Jul 11 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.8-1
- Update to 2.8
- Packaged only in EPEL7 and Fedora

* Sun Apr 26 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.7b1-2.20150420rev751
- Packaged license files
- Fixed library permissions

* Sat Apr 25 2015 Antonio Trande <sagitterATfedoraproject.org> - 2.7b1-1.20150420rev751
- initial build
